/*******************************************************
* Singleton.h
*
* Written by:     Ryan Fogarty
* Last Edit Date: 12-07-04
* Copyright:      2004
*
* Comments: 
*   As published in GEMs, algorithm by Scott Bilas 2000
*   with small modifications by me, Ryan Fogarty 2004
********************************************************/
#ifndef __SINGLETONCLASS__
#define __SINGLETONCLASS__

#include <iostream>

// NonCopyable Class contributed by Dave Abrahams
template<typename T>
class noncopyable
{
protected:
   noncopyable(){}
private:  // emphasize the following members are private
   noncopyable( const noncopyable& ) {}
   const noncopyable& operator=( const noncopyable& ) {return *this;}
}; 

template <typename T> class Singleton : public noncopyable<T>
{
private:
   static T* theSingleton;
protected:
   Singleton( void )
   {
      //theSingleton = static_cast <T*> (this);
   }
public:
   ~Singleton()
   {
      theSingleton = NULL;
   }
   static T& instance()
   {
      if(theSingleton == NULL)
      {
         create();
      }
      return ( *theSingleton );
   }
   static T* instancePtr()
   {
      return ( theSingleton );
   }
   static void create()
   {
      if(theSingleton == NULL)
      {
         theSingleton = new T();
      }
   }
   static void destroy()
   {
      if(theSingleton != NULL)
      {
         delete theSingleton; 
         theSingleton = NULL;
      }
   }
};

// Initialize the static data in the template class!
template <typename T> T* Singleton <T>::theSingleton = NULL;

#endif
